import { initDb } from '.';
import uri from '../lib/ids';
import { getFamily } from './lib';
import { putNewRte } from './rte';
import { putNewRtept } from './rtept';
import { putNewTrk } from './trk';
import { putNewTrkpt } from './trkpt';
import { putNewTrkseg } from './trkseg';
import { putNewWpt } from './wpt';

declare global {
  var db: any;
}

const originalDateNow = globalThis.Date.now;

describe('getFamily', () => {
  beforeEach(async () => {
    await initDb({});
    globalThis.Date.now = () => 0;
  });
  afterEach(async () => {
    await db.destroy();
    db = undefined;
    globalThis.Date.now = originalDateNow;
  });
  test('returns two rows after a gpx and a track have been inserted.', async () => {
    await putNewTrk();
    const allDocs: any = await getFamily(uri('gpx', { gpx: '' }));
    expect(allDocs).toMatchInlineSnapshot(`
Object {
  "offset": 0,
  "rows": Array [
    Object {
      "id": "gpx/1970-01-01T00%3A00%3A00.000Z",
      "key": "gpx/1970-01-01T00%3A00%3A00.000Z",
      "value": Object {
        "rev": "1-649694b1e46839ec63d2a93f483dd023",
      },
    },
    Object {
      "id": "gpx/1970-01-01T00%3A00%3A00.000Z/2trk/00000",
      "key": "gpx/1970-01-01T00%3A00%3A00.000Z/2trk/00000",
      "value": Object {
        "rev": "1-e1e6888ad978a5fcc405a75296604bee",
      },
    },
  ],
  "total_rows": 5,
}
`);
  });
  test('also returns the docs if required.', async () => {
    await putNewTrk();
    const allDocs: any = await getFamily(uri('gpx', { gpx: '' }), {
      include_docs: true,
    });
    expect(allDocs).toMatchInlineSnapshot(`
Object {
  "offset": 0,
  "rows": Array [
    Object {
      "doc": Object {
        "_id": "gpx/1970-01-01T00%3A00%3A00.000Z",
        "_rev": "1-649694b1e46839ec63d2a93f483dd023",
        "doc": Object {
          "$": Object {
            "creator": "dyomedea version 0.000002",
            "version": "1.1",
            "xmlns": "http://www.topografix.com/GPX/1/1",
            "xmlns:dyo": "http://xmlns.dyomedea.com/",
            "xmlns:gpxtpx": "http://www.garmin.com/xmlschemas/TrackPointExtension/v1",
            "xmlns:gpxx": "http://www.garmin.com/xmlschemas/GpxExtensions/v3",
            "xmlns:wptx1": "http://www.garmin.com/xmlschemas/WaypointExtension/v1",
            "xmlns:xsi": "http://www.w3.org/2001/XMLSchema-instance",
            "xsi:schemaLocation": "http://www.topografix.com/GPX/1/1 http://www.topografix.com/GPX/1/1/gpx.xsd http://www.garmin.com/xmlschemas/GpxExtensions/v3 http://www8.garmin.com/xmlschemas/GpxExtensionsv3.xsd http://www.garmin.com/xmlschemas/WaypointExtension/v1 http://www8.garmin.com/xmlschemas/WaypointExtensionv1.xsd http://www.garmin.com/xmlschemas/TrackPointExtension/v1 http://www.garmin.com/xmlschemas/TrackPointExtensionv1.xsd",
          },
          "metadata": Object {
            "time": "1970-01-01T00:00:00.000Z",
          },
        },
        "type": "gpx",
      },
      "id": "gpx/1970-01-01T00%3A00%3A00.000Z",
      "key": "gpx/1970-01-01T00%3A00%3A00.000Z",
      "value": Object {
        "rev": "1-649694b1e46839ec63d2a93f483dd023",
      },
    },
    Object {
      "doc": Object {
        "_id": "gpx/1970-01-01T00%3A00%3A00.000Z/2trk/00000",
        "_rev": "1-e1e6888ad978a5fcc405a75296604bee",
        "doc": Object {
          "number": 0,
        },
        "type": "trk",
      },
      "id": "gpx/1970-01-01T00%3A00%3A00.000Z/2trk/00000",
      "key": "gpx/1970-01-01T00%3A00%3A00.000Z/2trk/00000",
      "value": Object {
        "rev": "1-e1e6888ad978a5fcc405a75296604bee",
      },
    },
  ],
  "total_rows": 5,
}
`);
  });
  test('returns three rows after a gpx and a track segment have been inserted.', async () => {
    await putNewTrkseg();
    const allDocs: any = await getFamily(uri('gpx', { gpx: '' }));
    expect(allDocs).toMatchInlineSnapshot(`
Object {
  "offset": 0,
  "rows": Array [
    Object {
      "id": "gpx/1970-01-01T00%3A00%3A00.000Z",
      "key": "gpx/1970-01-01T00%3A00%3A00.000Z",
      "value": Object {
        "rev": "1-649694b1e46839ec63d2a93f483dd023",
      },
    },
    Object {
      "id": "gpx/1970-01-01T00%3A00%3A00.000Z/2trk/00000",
      "key": "gpx/1970-01-01T00%3A00%3A00.000Z/2trk/00000",
      "value": Object {
        "rev": "1-e1e6888ad978a5fcc405a75296604bee",
      },
    },
    Object {
      "id": "gpx/1970-01-01T00%3A00%3A00.000Z/2trk/00000/00000",
      "key": "gpx/1970-01-01T00%3A00%3A00.000Z/2trk/00000/00000",
      "value": Object {
        "rev": "1-9d8fbcd9cb7d2a45d6caf57a2cbee7ca",
      },
    },
  ],
  "total_rows": 6,
}
`);
  });
  test('returns four rows after a gpx and a track point have been inserted.', async () => {
    await putNewTrkpt();
    const allDocs: any = await getFamily(uri('gpx', { gpx: '' }));
    expect(allDocs).toMatchInlineSnapshot(`
Object {
  "offset": 0,
  "rows": Array [
    Object {
      "id": "gpx/1970-01-01T00%3A00%3A00.000Z",
      "key": "gpx/1970-01-01T00%3A00%3A00.000Z",
      "value": Object {
        "rev": "1-649694b1e46839ec63d2a93f483dd023",
      },
    },
    Object {
      "id": "gpx/1970-01-01T00%3A00%3A00.000Z/2trk/00000",
      "key": "gpx/1970-01-01T00%3A00%3A00.000Z/2trk/00000",
      "value": Object {
        "rev": "1-e1e6888ad978a5fcc405a75296604bee",
      },
    },
    Object {
      "id": "gpx/1970-01-01T00%3A00%3A00.000Z/2trk/00000/00000",
      "key": "gpx/1970-01-01T00%3A00%3A00.000Z/2trk/00000/00000",
      "value": Object {
        "rev": "1-9d8fbcd9cb7d2a45d6caf57a2cbee7ca",
      },
    },
    Object {
      "id": "gpx/1970-01-01T00%3A00%3A00.000Z/2trk/00000/00000/00000",
      "key": "gpx/1970-01-01T00%3A00%3A00.000Z/2trk/00000/00000/00000",
      "value": Object {
        "rev": "1-42b858110cde8bec09e5d34b9e480936",
      },
    },
  ],
  "total_rows": 7,
}
`);
  });

  test('returns two rows after a gpx and a waypoint have been inserted.', async () => {
    await putNewWpt();
    const allDocs: any = await getFamily(uri('gpx', { gpx: '' }));
    expect(allDocs).toMatchInlineSnapshot(`
Object {
  "offset": 0,
  "rows": Array [
    Object {
      "id": "gpx/1970-01-01T00%3A00%3A00.000Z",
      "key": "gpx/1970-01-01T00%3A00%3A00.000Z",
      "value": Object {
        "rev": "1-649694b1e46839ec63d2a93f483dd023",
      },
    },
    Object {
      "id": "gpx/1970-01-01T00%3A00%3A00.000Z/0wpt/00000",
      "key": "gpx/1970-01-01T00%3A00%3A00.000Z/0wpt/00000",
      "value": Object {
        "rev": "1-f98b9048879fc7b97771bf2bafc77e42",
      },
    },
  ],
  "total_rows": 5,
}
`);
  });

  test('returns two rows after a gpx and a route have been inserted.', async () => {
    await putNewRte();
    const allDocs: any = await getFamily(uri('gpx', { gpx: '' }));
    expect(allDocs).toMatchInlineSnapshot(`
Object {
  "offset": 0,
  "rows": Array [
    Object {
      "id": "gpx/1970-01-01T00%3A00%3A00.000Z",
      "key": "gpx/1970-01-01T00%3A00%3A00.000Z",
      "value": Object {
        "rev": "1-649694b1e46839ec63d2a93f483dd023",
      },
    },
    Object {
      "id": "gpx/1970-01-01T00%3A00%3A00.000Z/1rte/00000",
      "key": "gpx/1970-01-01T00%3A00%3A00.000Z/1rte/00000",
      "value": Object {
        "rev": "1-f17cd2939d1e63764478613f54835884",
      },
    },
  ],
  "total_rows": 5,
}
`);
  });

  test('returns three rows after a gpx and a route point have been inserted.', async () => {
    await putNewRtept();
    const allDocs: any = await getFamily(uri('gpx', { gpx: '' }));
    expect(allDocs).toMatchInlineSnapshot(`
Object {
  "offset": 0,
  "rows": Array [
    Object {
      "id": "gpx/1970-01-01T00%3A00%3A00.000Z",
      "key": "gpx/1970-01-01T00%3A00%3A00.000Z",
      "value": Object {
        "rev": "1-649694b1e46839ec63d2a93f483dd023",
      },
    },
    Object {
      "id": "gpx/1970-01-01T00%3A00%3A00.000Z/1rte/00000",
      "key": "gpx/1970-01-01T00%3A00%3A00.000Z/1rte/00000",
      "value": Object {
        "rev": "1-f17cd2939d1e63764478613f54835884",
      },
    },
    Object {
      "id": "gpx/1970-01-01T00%3A00%3A00.000Z/1rte/00000/00000",
      "key": "gpx/1970-01-01T00%3A00%3A00.000Z/1rte/00000/00000",
      "value": Object {
        "rev": "1-ba3f25be70b620dd06b453ec29853bae",
      },
    },
  ],
  "total_rows": 6,
}
`);
  });
});
